@@ -1,15 +1,20 @@
-# $Id: Hold.pm,v 1.12 2005/10/13 12:15:29 wsnyder Exp $
+# $Id: Hold.pm,v 1.2 2003/04/28 15:56:42 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2003 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::Hold;
@@ -24,7 +29,7 @@ use Carp;
 ######################################################################
 #### Configuration Section
 
-$VERSION = '2.022';
+$VERSION = '3.001';
 
 ######################################################################
 #### Creators
@@ -106,7 +111,7 @@ Schedule::Load::Hold - Return hold/wait information
 
 This package provides accessors for information about a specific request
 that is either waiting for a host, or has obtained a host and is holding it
-temporarily.
+temporarially.
 
 =head1 ACCESSORS
 
@@ -159,20 +164,16 @@ each unique request.
 
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<Schedule::Load::Hosts>, C<Schedule::Load::Hosts::Host>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<Schedule::Load::Hosts>, L<Schedule::Load::Hosts::Host>
-
 =cut
