@@ -1,15 +1,20 @@
-# $Id: ResourceReq.pm,v 1.12 2005/10/13 12:15:30 wsnyder Exp $
+# $Id: ResourceReq.pm,v 1.1 2003/04/24 13:14:11 wsnyder Exp $
 ######################################################################
 #
-# Copyright 2000-2004 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
+# This program is Copyright 2003 by Wilson Snyder.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of either the GNU General Public License or the
+# Perl Artistic License.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
+# If you do not have a copy of the GNU General Public License write to
+# the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
+# MA 02139, USA.
 ######################################################################
 
 package Schedule::Load::ResourceReq;
@@ -24,7 +29,7 @@ use Carp;
 ######################################################################
 #### Configuration Section
 
-$VERSION = '2.022';
+$VERSION = '3.001';
 
 ######################################################################
 #### Creators
@@ -107,14 +112,10 @@ prevents nasty deadlocks (like the chopsticks deadlock.)
 
 =head1 METHODS
 
-=over 4
-
 =item new (...)
 
 Create a new object with the parameters specified in the following section.
 
-=back
-
 =head1 PARAMETERS
 
 The following parameters are accepted by new(), and are also may be read
@@ -138,7 +139,7 @@ there are disk access time benefits to doing so.  Defaults to the current host.
 
 =item jobs_running
 
-Current number of jobs the requester is running.  This is compared to max_jobs.
+Current number of jobs the requestor is running.  This is compared to max_jobs.
 
 =item match_cb
 
@@ -163,22 +164,24 @@ host may not be used.  Ratings closer to zero are better.  Defaults to a
 function that includes the load_limit and the cpu percentage free.
 Evaluated in a Safe container, and can do only minimal core functions.
 
+=item
+
+=item
+
+=item
+
 =back
 
+=head1 SEE ALSO
+
+C<Schedule::Load>, C<Schedule::Load::Hosts>, C<Schedule::Load::Hosts::Host>
+
 =head1 DISTRIBUTION
 
-The latest version is available from CPAN and from L<http://www.veripool.com/>.
-
-Copyright 1998-2004 by Wilson Snyder.  This package is free software; you
-can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+The latest version is available from CPAN.
 
 =head1 AUTHORS
 
 Wilson Snyder <wsnyder@wsnyder.org>
 
-=head1 SEE ALSO
-
-L<Schedule::Load>, L<Schedule::Load::Hosts>, L<Schedule::Load::Hosts::Host>
-
 =cut
